#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <ftdi.h>

#define RELAY_1 1
#define RELAY_2 2
#define RELAY_3 3
#define RELAY_4 4

int main(int argc, char **argv)
{
    struct ftdi_context *ftdi;
    int f, opt;
    unsigned int relay = 0, state = 0;
    unsigned char buf[1];
    int retval = 0;

    openlog("relay_switch", LOG_CONS | LOG_PID | LOG_NDELAY, LOG_LOCAL1);

    while((opt = getopt(argc, argv, "r:s:")) != -1) {
        switch(opt) {
        case 'r':
            relay = atoi(optarg);
            if((relay != RELAY_1) && (relay != RELAY_2) && (relay != RELAY_3) && (relay != RELAY_4)) {
                syslog(LOG_ERR, "Invalid relay choosen (%d). Choose one of [1..4].\n", relay);
                return 1;
            }
            break;
        case 's':
            state = atoi(optarg);
            if((state != 0) && (state != 1)) {
                syslog(LOG_ERR, "Invalid state choosen (%d). Choose one of [0/1].\n", state);
                return 1;
            }
            break;
        }
    }

    if ((ftdi = ftdi_new()) == 0)
    {
        syslog(LOG_ERR, "ftdi_new failed\n");
        return EXIT_FAILURE;
    }

    f = ftdi_usb_open(ftdi, 0x0403, 0x6001);

    if (f < 0 && f != -5) {
        syslog(LOG_ERR, "unable to open ftdi device: %d (%s)\n", f, ftdi_get_error_string(ftdi));
        retval = 1;
        goto done;
    }

    f = ftdi_set_bitmode(ftdi, 0xFF, BITMODE_BITBANG);
    if(f < 0) {
        syslog(LOG_ERR, "unable to set bit bang mode: %d (%s)\n", f, ftdi_get_error_string(ftdi));
        retval = 1;
        goto done;
    }

    f = ftdi_read_data(ftdi, buf, 1);
    if(f < 0) {
        syslog(LOG_ERR, "unable to readfrom ftdi device: %d (%s)\n", f, ftdi_get_error_string(ftdi));
        retval = 1;
        goto done;
    }

    if(state == 1) {
        buf[0] |= (char)(1 << (relay - 1));
    }
    else {
        buf[0] &= ~(char)(1 << (relay - 1));
    }

    f = ftdi_write_data(ftdi, buf, 1);
    if(f < 0) {
        syslog(LOG_ERR, "write failed for 0x%x, error %d (%s)\n",buf[0], f, ftdi_get_error_string(ftdi));
    }

    ftdi_usb_close(ftdi);
done:
    ftdi_free(ftdi);
    closelog();

    return retval;
}
